use clap::{Arg, Command};
use parse_int::parse;
use scrypt::scrypt;
use std::error::Error;
use std::fmt::Write;

fn printhex(bytes: &[u8]) -> String {
    let mut s = String::new();
    for b in bytes {
        write!(s, "{:02x}", b).unwrap();
    }
    s
}

fn do_ecb(m: &str, k: u32) {
    let mut x = m.as_bytes().to_vec();
    println!("b: ecb");
    println!("k: {:#02x}", k);
    println!("m: 0x{}", printhex(&x));
    scrypt::enc_ecb(&mut x, k);
    println!("c: 0x{}", printhex(&x));
    scrypt::dec_ecb(&mut x, k);
    println!("d: 0x{}", printhex(&x));
}

fn do_cbc(m: &str, k: u32, iv: u8) {
    let mut x = m.as_bytes().to_vec();
    println!("b: cbc");
    println!("k: {:#02x}", k);
    println!("i: {:#02x}", iv);
    println!("m: 0x{}", printhex(&x));
    scrypt::enc_cbc(&mut x, k, iv);
    println!("c: 0x{}", printhex(&x));
    scrypt::dec_cbc(&mut x, k, iv);
    println!("d: 0x{}", printhex(&x));
}

fn main() -> Result<(), Box<dyn Error>> {
    let matches = Command::new("scrypt")
	.version("0.1.0")
	.about("Simple substitution permutation crypto algorithm")
        .arg(Arg::new("key")
             .short('k')
             .default_value("0x98267351")
             .help("Symmetric key"))
        .arg(Arg::new("iv")
             .short('i')
             .default_value("0x42")
             .help("Initialization vector"))
        .arg(Arg::new("args")
             .multiple_values(true)
             .help("Strings to encrypt and decrypt"))
        .get_matches();

    let key: u32 = parse::<u32>(matches.value_of("key").unwrap())?;
    let iv: u8 = parse::<u8>(matches.value_of("iv").unwrap())?;

    let mut args = vec![];
    if let Some(values) = matches.values_of("args") {
	for v in values {
            args.push(v.to_string());
	}
    }

    for m in &args {
        do_ecb(m, key);
    }
    println!();
    for m in &args {
        do_cbc(m, key, iv);
    }

    Ok(())
}
